import yaml
from rich.console import Console
from rich.logging import RichHandler
from pathlib import Path
from random import randrange, seed
import logging
import os
import toml
from box import Box

# -----------------------------------------------------------------

logging.basicConfig(
    level="NOTSET",
    format="%(message)s",
    datefmt="[%X]",
    handlers=[RichHandler(rich_tracebacks=True)],
)

logger = logging.getLogger("rich")
console = Console()

# -----------------------------------------------------------------


def print(*args, **kwargs):
    console.print(*args, **kwargs)


# -----------------------------------------------------------------


def color_value(value: str):
    seed(value.lower())
    r, g, b = [str(hex(randrange(25, 255))[2:]) for _ in range(3)]
    value_colored = f"[bold #{r}{g}{b}]{value}[/]"

    return value_colored


# -----------------------------------------------------------------

CONFIG_HOME = Path(os.environ.get("XDG_CONFIG_HOME", "~/.config")).expanduser()
CONFIG_FILE = CONFIG_HOME / "novara" / "config.yml"
SSHKEY_FILE = CONFIG_HOME / "novara" / "novara.key"

def write_config(config: dict):
    try:
        dot_config = CONFIG_HOME
        if not dot_config.exists():
            logger.info(f"creating directory {dot_config}")
            dot_config.mkdir()
        config_directory = CONFIG_FILE.parent
        if not config_directory.exists():
            logger.info(f"creating directory {config_directory}")
            config_directory.mkdir()
        yaml.dump(config, open(CONFIG_FILE, "w"))
    except OSError:
        logger.error("Couldn't create the config file it's not writable")
        exit()
    # --------------
    try:
        with open(SSHKEY_FILE, "w") as f:
            f.write(config["ssh_privatekey"])
    except OSError:
        logger.error("Couldn't create the SSH-key it's not writable")
    # --------------
    try:
        os.chmod(SSHKEY_FILE, 0o600)
    except OSError:
        logger.error("Couldn't change the SSH-key's permissions")
        exit()

def get_current_config():
    try:
        with open("novara.toml", "r") as f:
            # exploit_config = toml.load(f)
            toml_parsed = toml.load(f)
    except (OSError, FileNotFoundError):
        return None
    return Box(toml_parsed)


import rich_click as click
from novara.utils import write_config
from urllib.parse import urljoin, urlparse
import os
import requests


@click.command()
@click.option(
    "--server_url",
    "-l",
    default=None,
    help="url to api-endpoint of your novara instance",
)
@click.option(
    "--username",
    "-u",
    default=None,
    help="username for basic auth. In the case of oauth this parameter does nothing.",
)
@click.option(
    "--password",
    "-p",
    default=None,
    help="password for basic auth. In the case of oauth this parameter does nothing.",
)
@click.option(
    "--author",
    "-a",
    default=None,
    help="to specify what author to use for the exploits",
)
def configure(server_url, username, password, author):
    """conect to novara backend & configure the cli"""

    # Priority: CLI argument > Environment variable > Prompt

    server_url = (
        server_url
        or os.environ.get("SERVER_URL")
        or click.prompt("Please enter the Novara server URL")
    )

    parsed_server_url = urlparse(server_url)

    username = (
        username
        or parsed_server_url.username
        or click.prompt("Please enter your username")
    )
    password = (
        password
        or parsed_server_url.password
        or os.environ.get("PASSWORD")
        or click.prompt("Please enter your password")
    )

    author = (
        author
        or os.environ.get("AUTHOR_NAME")
        or click.prompt("Please enter your author username")
    )

    # -----------------------------------------------------------------

    r = requests.get(
        urljoin(server_url, "/api/config/"),
        auth=requests.auth.HTTPBasicAuth(username, password),
    )
    if not r.ok:
        raise click.ClickException(f"the remote responded with error:\n{r.text}")
        exit()

    # -----------------------------------------------------------------

    try:
        config = r.json()
    except requests.JSONDecodeError:
        raise click.ClickException(f"unable to decode response as json:\n{r.text}")

    config["server_url"] = server_url
    config["author"] = author

    if config["auth_type"] == "basic":
        config["basic_auth"] = [username, password]

    write_config(config)



import rich_click as click
from novara.request import request, JSONDecodeError
from novara.utils import logger, get_current_config, console
from bullet import Bullet, YesNo
from io import BytesIO
from zipfile import ZipFile
import os
import shutil


@click.command()
@click.option("-e", "--exploit", default=None, help="specify a exploit id manually")
@click.option("-l", "--lines", default='50', help="specify the max amount of lines printed")
def logs(exploit, lines):
    """Request the logs of the exploit container"""
    config = get_current_config()
    if config is None:
        raise click.ClickException("novara.toml either not there or unaccessable")
    exploit_id = config.exploit_id
    r = request.get('/exploits/')
    if not r.ok:
        raise click.ClickException(
            f"failed requesting a list of exploits from remote with error: {r.text}"
        )
    try:
        exploits = r.json()
    except JSONDecodeError:
        raise click.ClickException(f"unable to decode response as json:\n{r.text}")

    current_exploit:dict|None = None

    for exploit in exploits:
        if exploit['id'] == exploit_id:
            current_exploit = exploit

    if current_exploit is None:
        raise click.ClickException(f'No exploit with id {exploit_id} found on remote')
    
    # console.print_json(data=current_exploit)
    logger.info(f'exploit: {current_exploit["service"]}-{current_exploit["name"]}')

    container_id = current_exploit.get('container_id')

    logger.info(f'container: {container_id}')

    if container_id is None:
        raise click.ClickException(f'No container found for exploit {exploit.get("name")}')
    
    r = request.get(f'containers/{container_id}/logs')
    if not r.ok:
        raise click.ClickException(
            f"failed requesting a logs of container {container_id} from remote with error: {r.text}"
        )
    try:
        logs:str = r.json()
    except JSONDecodeError:
        raise click.ClickException(f"unable to decode response as json:\n{r.text}")
    
    max_lines = int(lines)

    total_lines = logs.count("\n")

    logger.info(f'displaying {min(max_lines, total_lines)} out of {total_lines} lines')

    if total_lines > max_lines:
        logs = '\n'.join(logs.split('\n')[-max_lines:])

    click.echo(logs)


import rich_click as click
from novara.utils import color_value, print
from rich.table import Table, box
from novara.request import request


@click.command()
@click.option(
    "--container",
    "-c",
    default=None,
    help="specify container that the output should be filtered by",
)
def status(container):
    """get infos / status of currently running exploits"""

    r = request.get("containers/")
    if not r.ok:
        raise click.ClickException(
            f"requesting status from remote failed with error:\n{r.text}"
        )
        exit()

    status_infos = r.json()
    if (
        not any([status.get("container_id") == container for status in status_infos])
        and container is not None
    ):
        raise click.ClickException("No container with this id found")
        exit()

    # -----------------------------------------------------------------

    table = Table(title="container(s):", box=box.MINIMAL, highlight=True)

    table.add_column("Service:")
    table.add_column("Exploit:")
    table.add_column("Author:")
    table.add_column("Health:")
    table.add_column("Starttime:")
    table.add_column("Container:")

    for info in status_infos:
        if container != None and (container in info["id"]):
            ...
        else:
            col_health = "green" if info["health"] == "ok" else "red bold"

            table.add_row(
                f"[bold]{info['service']}[/]",
                f"[bold]{info['exploit']}[/]",
                f"{color_value(info['author'])}",
                f"[{col_health}]{info['health']}[/]",
                f"{info['start_time'].split('.')[0].replace('T', ' ')}",
                f"[bold]{info['id'][:12]}[/]",
            )

    print(table)



import rich_click as click
import subprocess
import os
from novara.utils import logger, SSHKEY_FILE
from novara.config import config


@click.command(
    context_settings=dict(
        ignore_unknown_options=True,
        allow_extra_args=True,
    )
)
@click.pass_context
def docker(ctx):
    """this command allows you to control the docker on the remote with the docker cli"""

    env = {
        **os.environ.copy(),
        "DOCKER_HOST": f"ssh://{config.ssh_user}@{config.ssh_url}:{config.ssh_port}",
        "DOCKER_SSH_KEYPATH": SSHKEY_FILE,
    }

    docker = subprocess.Popen(["docker", *ctx.args], env=env)

    try:
        docker.wait()
    except KeyboardInterrupt:
        logger.warn("terminating tunnel")
    docker.kill()



import rich_click as click
from novara.utils import logger
from novara.commands.generate import generate_dockerfile
import docker
import toml
from io import BytesIO
from zipfile import ZipFile
import os
from novara.request import request, JSONDecodeError
import json
import docker.models.containers


@click.group()
def run():
    """run a exploit script either locally or on remote"""


@run.command()
def local():
    """run the exploit script locally"""
    client = docker.from_env()

    generate_dockerfile()

    logger.info("building image")
    image, logs = client.images.build(path=".")
    for docker_log in logs:
        logger.info(f'logs: {docker_log.get("stream", str(docker_log)).strip()}')

    logger.info("starting container")
    container: docker.models.containers.Container = client.containers.run(
        image, detach=True
    )
    try:
        for docker_log in container.logs(stream=True):
            click.echo(docker_log, nl=False)
    except KeyboardInterrupt:
        logger.warn("stopping container")
        container.stop()


@run.command()
@click.option(
    "-k",
    "--keep",
    default=False,
    is_flag=True,
    help="keep container, if it already exists",
)
def remote(keep):
    """upload and run the exploit on the remote server"""

    try:
        with open("novara.toml", "r") as f:
            toml_config = toml.load(f)
    except (OSError, FileNotFoundError):
        raise click.ClickException("novara.toml either not there or unaccessable")
        exit()
    logger.info("read toml file")

    if "exploit_id" not in toml_config:
        raise click.ClickException(
            "no exploit_id in toml found, consider regenerating the toml with novara init"
        )
        exit()

    exploit_id = toml_config.get("exploit_id")

    zip_archive = BytesIO()

    with ZipFile(zip_archive, "w") as zip:
        for root, _, filenames in os.walk(os.getcwd()):
            for name in filenames:
                zip.write(name, name)

    zip_archive.seek(0)

    logger.info("uploading zip...")

    r = request.post(f"exploits/{exploit_id}/", files={"file": zip_archive})
    if not r.ok:
        raise click.ClickException(f"Uploading zip failed with error: {r.text}")
        exit()

    try:
        with open("novara.toml", "r") as toml_file:
            toml_str = toml_file.read()
    except (FileNotFoundError, OSError):
        raise click.ClickException("Failed reading novara toml file")
        exit()

    logger.info("building image...")

    r = request.post(f"build/{exploit_id}/toml/", data=json.dumps(toml_str))
    if not r.ok:
        raise click.ClickException(
            f"Failed building novara toml file with error: {r.text}"
        )
        exit()

    try:
        image = r.json()
    except JSONDecodeError:
        raise click.ClickException(
            f"failed to decode response as json: {r.text[:20] if len(r.text) > 20 else r.text}"
        )

    if image is None:
        raise click.ClickException("Something went wrong while building image")
        exit()

    logger.info(f"image: {image}")

    if not keep:
        logger.info(f"requesting removal of old containers of exploit: {exploit_id}")
        r = request.delete(f"exploits/{exploit_id}/")
        if r.status_code == 404:
            logger.info("No containers found for current exploit")
        if not r.ok:
            logger.warn(f"failed removing container with message: {r.text}")

    logger.info("starting new container...")

    r = request.post("containers/", params={"exploit_id": exploit_id})
    if not r.ok:
        raise click.ClickException(
            f"Failed starting new container with error: {r.text}"
        )
        exit()

    try:
        container_info = r.json()
    except JSONDecodeError:
        raise click.ClickException(
            f"failed to decode response as json: {r.text[:20] if len(r.text) > 20 else r.text}"
        )
    for info in container_info:
        logger.info(f"{info}: {container_info[info]}")

    logger.info(
        "Done deploying new container, to redeploy the container just run 'novara run remote' again"
    )



import rich_click as click
from novara.request import request, JSONDecodeError
from novara.utils import logger, get_current_config
from bullet import Bullet, YesNo
from io import BytesIO
from zipfile import ZipFile
import os
import shutil


def download_exploit(exploit_id: str, exploit_name: str, dir):
    if dir is None:
        dir = os.path.normpath(os.path.join(os.getcwd(), exploit_name))
    logger.info(f"exploit id: {exploit_id}")
    logger.info("requesting exploit")

    r = request.get(f"exploits/{exploit_id}/")
    if not r.ok:
        raise click.ClickException(
            f"Requesting exploit from remote failed with error: {r.text}. Did you run novara configure?"
        )
        exit()
    logger.info("extracting exploit")
    zip_template = BytesIO(r.content)
    if os.path.exists(dir):
        overwrite = YesNo(
            prompt=f"Do you want to overwrite the directory '{dir}'? (y/n)",
            prompt_prefix="",
            default="y",
        ).launch()
        if not overwrite:
            raise click.ClickException(
                "Directory already exists and can't be overwritten. Consider using the -d or --dir to specify a different directory."
            )
            exit()
        shutil.rmtree(dir)
    os.mkdir(dir)
    with ZipFile(zip_template) as zip:
        zip.extractall(dir)

    logger.info("Exploit extracted sucessfully")


@click.command()
@click.option(
    "-s", "--service", default=None, help="name of the service the exploit is targeting"
)
@click.option("-n", "--name", default=None, help="full name of the exploit to pull")
@click.option(
    "-d",
    "--directory",
    default=None,
    help="specify a different directory to save the exploit",
)
@click.option("-a", "--all", default=False, help="list all exploits", is_flag=True)
def pull(service, name, directory, all):
    """Pull existing exploits from novara, if no other arguments are provided and there is a novara.toml in the same directory the cli will try to pull the newest version of the exploit"""
    exploit = None
    if service is None and name is None and directory is None and not all:
        exploit = get_current_config()
    if exploit is not None:
        download_exploit(exploit['exploit_id'], f'{exploit['service']['name']}-{exploit['exploit_name']}', os.getcwd())
        exit()
    r = request.get("exploits/")
    if not r.ok:
        raise click.ClickException(
            f"failed requesting a list of exploits from remote with error: {r.text}"
        )
        exit()
    try:
        exploits = r.json()
    except JSONDecodeError:
        raise click.ClickException(f"unable to decode response as json:\n{r.text}")

    # -----------------------------------------------------------------

    # check if any exploits are even available
    if not len(exploits):
        raise click.ClickException("No exploits to pull!")
        exit()

    # pull by name ----------------------------------------------------
    if name and any(i["name"] == name for i in exploits):
        exploit = None
        for exploit in exploits:
            if exploit["name"] == name:
                exploit = exploit
                break

        download_exploit(exploit["id"], name, directory)
        exit()
    elif name:
        raise click.ClickException(f"No exploits named '{name}' available!")
        exit()

    # chose a service -------------------------------------------------

    if not all:
        services = []
        for exploit in exploits:
            if not exploit['service'] in services:
                services.append(exploit['service'])
        service = Bullet(
                prompt="please select a service:",
                choices=services,
            ).launch()

    # pull by service -------------------------------------------------
    if service and any(exploit["service"] == service for exploit in exploits):
        service_exploits = []
        for exploit in exploits:
            service_exploits.append(exploit) if exploit["service"] == service else None

        if len(service_exploits) > 1:
            name, index = Bullet(
                prompt="there are multiple exploits available for this service, please select one:",
                choices=[exploit["name"] for exploit in service_exploits],
                return_index=True,
            ).launch()
            exploit = service_exploits[index]
        else:
            exploit = service_exploits[0]

        download_exploit(exploit["id"], f'{exploit['service']}-{exploit['name']}', directory)
        exit()

    elif name:
        raise click.ClickException(f"No exploits for '{service}' available")
        exit()

    # pull by bullet-selection ----------------------------------------
    else:
        name, index = Bullet(
            prompt="chose a exploit:",
            choices=[f'{exploit["service"]}-{exploit["name"]}' for exploit in exploits],
            return_index=True,
        ).launch()

        exploit = exploits[index]
        download_exploit(exploit["id"], name, directory)



import rich_click as click
from string import ascii_lowercase, digits
from novara.utils import logger
from novara.request import request, JSONDecodeError
from random import choices
from io import BytesIO
from bullet import Bullet, YesNo
from zipfile import ZipFile
import os
import shutil
from novara.config import config


def get_service():
    r = request.get("services/")
    if not r.ok:
        raise click.ClickException(
            f"Failed requesting list of services from remote with error: {r.text}"
        )
        exit()
    try:
        services = r.json()
    except JSONDecodeError:
        raise click.ClickException(
            f"failed to decode response as json: {r.text[:20] if len(r.text) > 20 else r.text}"
        )

    service = Bullet(prompt="Please select a service", choices=services).launch()

    return service


@click.command()
@click.option(
    "-s",
    "--service",
    default=None,
    help="the name of the service the exploit will be attacking",
)
@click.option(
    "-n", "--name", default=None, help="the internal name for the exploit identifing it"
)
@click.option("-a", "--author", default=None, help="name of the exploit's author")
@click.option(
    "-d",
    "--directory",
    default=None,
    help="specify a different directory to put the exploit",
)
def init(service, name, author, directory):
    """Initialize a new exploit from a template"""

    # Priority: CLI argument > Environment variable > Prompt

    service = service or get_service()
    name = name or "".join(choices(ascii_lowercase + digits, k=6))
    author = (
        author
        or config.author
        or os.environ.get("AUTHOR")
        or click.prompt("Please enter this exploit author's name")
    )

    # -----------------------------------------------------------------

    full_directory = os.path.join(os.getcwd(), directory or f"{service}-{name}")
    logger.info(f"setting up directory: {full_directory}")

    if os.path.exists(full_directory):
        if YesNo(f"Do you want to overwrite the directory '{full_directory}'? (y/n)"):
            shutil.rmtree(full_directory)
        else:
            logger.info("Directory won't be overwritten, exiting...")
            exit()

    logger.info("requesting template")

    r = request.post(
        f"services/{service}/template/",
        params={"exploit_name": name, "exploit_author": author, "additional_args": ""},
    )
    if not r.ok:
        raise click.ClickException(
            f"Requesting template from remote failed with error: {r.text}. Did you run novara configure?"
        )
        exit()

    logger.info("extracting template")
    zip_template = BytesIO(r.content)
    os.mkdir(full_directory)
    with ZipFile(zip_template) as zip:
        zip.extractall(full_directory)

    logger.info(f"Template extracted sucessfully into directory {directory or f"{service}-{name}"}")
    logger.info("To add a new dependency run 'novara generate'")
    logger.info("To run the current exploit run 'novara run [local|remote]'")
    logger.info("Happy exploiting!")



import requests
from novara.config import config
from urllib.parse import urljoin
import requests.auth
import click
from requests import JSONDecodeError

class request:
    @staticmethod
    def get(*args, **kwargs):
        return request.request("get", *args, **kwargs)

    @staticmethod
    def post(*args, **kwargs):
        return request.request("post", *args, **kwargs)

    @staticmethod
    def delete(*args, **kwargs):
        return request.request("delete", *args, **kwargs)

    def request(type, path: str, *args, **kwargs):
        try:
            if not path.endswith("/"):
                path += "/"
            if not path.startswith("/api/"):
                path = ("/api/" + path).replace("//", "/")
            return requests.request(
                *args,
                method=type,
                auth=requests.auth.HTTPBasicAuth(*config.basic_auth)
                if config.auth_type == "basic"
                else None,
                url=urljoin(config.server_url, path),
                **kwargs,
            )
        except Exception as e:
            raise click.ClickException(
                f"request to remote failed with error {e}. Did you run novara configure?"
            )


from novara.utils import CONFIG_FILE
from box import Box
import yaml
import click

class Config:
    conf = None
    def _load(self):
        try:
            with open(CONFIG_FILE, "r") as f:
                conf_dict = yaml.safe_load(f)
            self.conf = Box(conf_dict)
        except:
            raise click.ClickException("""*Unable to load config file!!!* -> This will break the functionality of most commands
            Hint: You can create a new config file using 'novara configure'""")

    def __getattr__(self, name: str):
        if self.conf is None:
            self._load()
        return self.conf.__getattr__(name)

config = Config()